#!/usr/bin/env python3

# PODCAST HANDLER by Claudio Barca (Copyright 2020 Claudio Barca)
# This software is distributed under GPL v. 3 license.
# See LICENSE file for details.

import traceback
import os
import datetime
import time
from mpd import MPDClient
import curses
from curses import wrapper

from podcasthnd.media import Media
from podcasthnd.item import Item
import podcasthnd.constants as constants

class WinMain:  # main window
    def __init__(self):
        begin_x = 2; begin_y = 2
        height = 20; width = (curses.COLS - 4)
        self.win = curses.newwin(height, width, begin_y, begin_x)

    def create(self):
        global item, media
        self.win.clear()
        self.win.addstr(1,   5, str('PODCAST HANDLER ver. ' + constants.version), curses.A_BOLD)
        self.win.addstr(3,   5, str('Host: ' + media.host), curses.A_BOLD)
        self.win.addstr(4,   5, str('Item: ' + item.url), curses.A_BOLD)

        self.win.addstr( 7,  5, 'Press p to toggle play/pause', curses.A_NORMAL)
        self.win.addstr( 8,  5, 'Press < to seek -10 secs', curses.A_NORMAL)
        self.win.addstr( 9,  5, 'Press > to seek +30 secs', curses.A_NORMAL)
        self.win.addstr(10,  5, 'Press s to seek for a position', curses.A_NORMAL)
        
        self.win.addstr(13,  5, 'Press r to restart item', curses.A_NORMAL)
        self.win.addstr(15,  5, 'Press b to close this screen and play in background', curses.A_NORMAL)
        self.win.addstr(16,  5, 'Press q to close this screen and stop audio', curses.A_NORMAL)
        
        if media.notseekable_flag:
            self.win.addstr( 5,12, '(Not seekable)', curses.A_BOLD)
        self.win.refresh()

    def get_value(self,question,line):  # prompt for a generic new value
        global media
        self.win.addstr(line, 5, question , curses.A_BOLD)
        self.win.refresh()
        self.win.nodelay(False)
        curses.echo()
        answer = self.win.getstr(line, 27 , 6)
        curses.noecho()
        self.win.nodelay(True)
        self.win.clear()
        self.create()
        return str(answer)[2:-1]

    def new_position(self):  # set new position
        question = 'Insert time (mm:ss):'
        line = 19               # line number of the text 
        s = self.get_value(question, line).split(':')
        hours   = int(str(s[0]))
        minutes = int(str(s[1]))
        newpos = (hours * 60) + minutes
        media.seek(newpos)

    def loop(self):
        global item, media, bar
        while True:
            self.win.nodelay(True)
            ch = self.win.getch()

            if ch == ord('q'):
                media.close()  #  stop and close mpd connectio
                break
            
            if ch == ord('b'):
                media.client.close()  # close mpd connection
                break

            if ch == ord('p'):
                media.toggle()
                self.win.refresh()

            if ch == ord('<'):
                media.seek('-10')
                self.win.refresh()

            if ch == ord('>'):
                media.seek('+30')
                self.win.refresh()

            if ch == ord('s'):  # get new position
                self.new_position()
                self.win.refresh()

            if ch == ord('r'):  # restart podcast
                media.client.play()
                media.seek('1')

            if media.client.status()['state'] == "play":
                my_time = media.time()
                bar.update(my_time[0], my_time[1])

            elif media.client.status()['state'] == "stop":
                media.client.close()
                break

            time.sleep(0.9)

class WinBar:  #  progress bar window
    def __init__(self):
        self.begin_x = 2 
        self.begin_y = (curses.LINES - 4)
        self.height = 3
        self.width = (curses.COLS - 4)
        self.win = curses.newwin(self.height, self.width, self.begin_y, self.begin_x)
        self.win.border(0)

    def create(self):
        self.win.refresh()

    def update(self, position, total):
        try:
            rangex = ((int(self.width - 20 ) * (int(position) + 1)) / int(total))
        except:
            rangex = 1
        pos = int(rangex) + 9
        for pp in range(9,pos):
            self.win.addstr(1, pp, "=")
        self.win.addstr(1, 1, str(datetime.timedelta(seconds=int(position))))
        self.win.addstr(1, (self.width -9 ), str(datetime.timedelta(seconds=int(total))))
        self.win.addstr(1, pos, ">")
        self.win.refresh()

def main(stdscr):
    global bar
    bar = WinBar()
    bar.create()
    info = WinMain()
    info.create()
    info.loop()

def start_gui(host):
    global media, item
    media = Media(host)
    item = Item(media.client.currentsong()['file'])
    wrapper(main)



#!/usr/bin/env python3

# PODCAST HANDLER by Claudio Barca (Copyright 2020 Claudio Barca)
# This software is distributed under GPL v. 3 license.
# See LICENSE file for details.

# class to handle mpd

from mpd import MPDClient

class Media:
    def __init__(self, host):
        from mpd import MPDClient
        self.client = MPDClient()  
        self.host = host
        self.client.connect(self.host, 6600)  
        self.notseekable_flag = False

    def play(self,url):         # set and play url
        self.url = url
        self.client.clear()
        self.client.repeat(0)
        self.client.random(0)
        self.client.add(url)
        self.client.play()

    def play_at_position(self,url,position):  # start playing an url at position
        self.play(url)
        self.seek(position)

    def toggle(self):           # toggle pause
        self.client.pause()

    def seek(self,position):    # seek for a position
        try:
            self.client.seekcur(position)
            self.notseekable_flag = False
        except:
            self.notseekable_flag = True

    def time(self):             # return a list [ current seconds, total seconds ]
        return self.client.status()['time'].split(':')

    def close(self):            # stop playing and close mpd connection
        self.client.stop()
        self.client.close()



#!/usr/bin/env python3

# PODCAST HANDLER by Claudio Barca (Copyright 2020 Claudio Barca)
# This software is distributed under GPL v. 3 license.
# See LICENSE file for details.

import os

# constants

version = "0.6.1"

podcast_cache_dir = ("%s/.cache/podcasthander/" % os.environ['HOME'])
daemon_data_file  = podcast_cache_dir + "daemon_data"
current_file = podcast_cache_dir + "current"
daemon_filename   = "podcasthandlerd"
default_host      = "localhost"

update_time = 10                # seconds between update 
end_time = 15                   # seconds before the end of file to stop updating and delete item



#!/usr/bin/env python3

import setuptools 

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='podcasthandler',
    version='0.6.1',
    description='Easily play podcasts',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fnt400/podcasthandler",
    author='Claudio Barca',
    author_email='claudio@barca.mi.it',
    scripts=['podcasthandler','podcasthandlerd'],
    packages=['podcasthnd'],
    classifiers=[
       "Programming Language :: Python :: 3",
       "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
       "Operating System :: OS Independent",
    ],
    install_requires=["python-mpd2 ~= 1.0.0"],
    )



#!/usr/bin/env python3

# PODCAST HANDLER by Claudio Barca (Copyright 2020 Claudio Barca)
# This software is distributed under GPL v. 3 license.
# See LICENSE file for details.

import os
import hashlib
import podcasthnd.constants as constants

class Item:
    def __init__(self,url):
        self.url = url
        self.hash = self.get_hash_string()
        self.cache_file = constants.podcast_cache_dir + self.hash    # save the position of a podcast
        self.current_file = constants.current_file

    def db_get_position(self):
        try:
            file = open(self.cache_file,'r') 
            position = int(file.readlines()[0])
            return position
        except:
            return 0

    def db_set_position(self,position):
        file = open(self.cache_file,'w') 
        file.write(position)
        file.close() 

    def db_set_current(self):   # save the current podcast url
        file = open(self.current_file,'w') 
        file.write(self.url)
        file.close() 

    def db_delete_cache_file(self):
        try:
            os.remove(self.cache_file)
        except:
            print('No cache file')

    # the cache filename is a sha256 hash of the url text
    def get_hash_string(self):
        hash = hashlib.sha256(self.url.encode('utf-8')).hexdigest()
        return hash


